//
//  GLView.m
//  ShaderExamples
//
//  Created by Brad Larson on 11/13/2010.
//

#import "GLView.h"
#import <OpenGLES/EAGLDrawable.h>
#import <QuartzCore/QuartzCore.h>

//#define MSAA 1

@implementation GLView

#pragma mark -
#pragma mark Initialization and teardown

+ (Class) layerClass 
{
	return [CAEAGLLayer class];
}


- (id)initWithCoder:(NSCoder *)decoder
{
    self = [super initWithCoder:decoder];
    if (self) 
	{
        // Initialization code.
		CAEAGLLayer *eaglLayer = (CAEAGLLayer *)self.layer;
		eaglLayer.opaque = YES;
		eaglLayer.drawableProperties = [NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithBool:NO], kEAGLDrawablePropertyRetainedBacking, kEAGLColorFormatRGBA8, kEAGLDrawablePropertyColorFormat, nil];		
		context = [[EAGLContext alloc] initWithAPI:kEAGLRenderingAPIOpenGLES2];
		
		if (!context || ![EAGLContext setCurrentContext:context] || ![self createFramebuffers]) 
		{
			NSLog(@"Failed creating context");
			[self release];
			return nil;
		}
		
	}
	
	return self;
}

#pragma mark -
#pragma mark OpenGL drawing

- (BOOL)createFramebuffers;
{	
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_DEPTH_TEST);
	
	// Offscreen framebuffer object
	glGenFramebuffers(1, &offscreenFramebuffer);
    glBindFramebuffer(GL_FRAMEBUFFER, offscreenFramebuffer);
	
	glGenRenderbuffers(1, &offscreenRenderbuffer);
    glBindRenderbuffer(GL_RENDERBUFFER, offscreenRenderbuffer);
	
    glRenderbufferStorage(GL_RENDERBUFFER, GL_RGBA8_OES, 512, 512);
    glFramebufferRenderbuffer(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, GL_RENDERBUFFER, offscreenRenderbuffer);	

	glGenRenderbuffers(1, &offscreenDepthBuffer);
	glBindRenderbuffer(GL_RENDERBUFFER, offscreenDepthBuffer);
	glRenderbufferStorage(GL_RENDERBUFFER, GL_DEPTH_COMPONENT16, 512, 512);
	glFramebufferRenderbuffer(GL_FRAMEBUFFER, GL_DEPTH_ATTACHMENT, GL_RENDERBUFFER, offscreenDepthBuffer);
	
	// Offscreen framebuffer texture target
	glGenTextures(1, &offscreenRenderTexture);
	glBindTexture(GL_TEXTURE_CUBE_MAP, offscreenRenderTexture);
	glTexParameteri( GL_TEXTURE_CUBE_MAP,GL_TEXTURE_MIN_FILTER, GL_LINEAR );
	glTexParameteri( GL_TEXTURE_CUBE_MAP,GL_TEXTURE_MAG_FILTER, GL_LINEAR );
	glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	for (int f = 0; f < 6; ++f) {
        GLenum face = GL_TEXTURE_CUBE_MAP_POSITIVE_X + f;
        glTexImage2D(face, 0, GL_RGBA, 512, 512, 0, GL_RGBA, GL_UNSIGNED_BYTE, 0);
    }
	
//	glFramebufferTexture2D(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, GL_TEXTURE_CUBE_MAP, offscreenRenderTexture, 0);
	
	
	GLenum status = glCheckFramebufferStatus(GL_FRAMEBUFFER);
    if (status != GL_FRAMEBUFFER_COMPLETE) 
	{
		NSLog(@"Incomplete FBO: %d", status);
        exit(1);
    }
	
	// Onscreen framebuffer object
	glGenFramebuffers(1, &viewFramebuffer);
	glBindFramebuffer(GL_FRAMEBUFFER, viewFramebuffer);
	
	glGenRenderbuffers(1, &viewRenderbuffer);
	glBindRenderbuffer(GL_RENDERBUFFER, viewRenderbuffer);
	
	[context renderbufferStorage:GL_RENDERBUFFER fromDrawable:(CAEAGLLayer*)self.layer];
	
	glGetRenderbufferParameteriv(GL_RENDERBUFFER, GL_RENDERBUFFER_WIDTH, &backingWidth);
	glGetRenderbufferParameteriv(GL_RENDERBUFFER, GL_RENDERBUFFER_HEIGHT, &backingHeight);
	
	glFramebufferRenderbuffer(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, GL_RENDERBUFFER, viewRenderbuffer);
	
#ifdef MSAA
	// Multisampled antialiasing
	glGenFramebuffers(1, &msaaFramebuffer); 
	glGenRenderbuffers(1, &msaaRenderbuffer);
	
	glBindFramebuffer(GL_FRAMEBUFFER, msaaFramebuffer); 
	glBindRenderbuffer(GL_RENDERBUFFER, msaaRenderbuffer);   
	
	// 4X MSAA
	glRenderbufferStorageMultisampleAPPLE(GL_RENDERBUFFER, 4, GL_RGBA8_OES, backingWidth, backingHeight); 
	glFramebufferRenderbuffer(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, GL_RENDERBUFFER, msaaRenderbuffer); 

	glGenRenderbuffers(1, &msaaDepthbuffer);   
	glBindRenderbuffer(GL_RENDERBUFFER, msaaDepthbuffer); 
	glRenderbufferStorageMultisampleAPPLE(GL_RENDERBUFFER, 4, GL_DEPTH_COMPONENT16, backingWidth, backingHeight); 
	glFramebufferRenderbuffer(GL_FRAMEBUFFER, GL_DEPTH_ATTACHMENT, GL_RENDERBUFFER, msaaDepthbuffer);
	
	if (glCheckFramebufferStatus(GL_FRAMEBUFFER) != GL_FRAMEBUFFER_COMPLETE)
	{
		NSLog(@"Failed to make complete framebuffer object %x", glCheckFramebufferStatus(GL_FRAMEBUFFER));
		return NO;
		
	}
#else
	glGenRenderbuffers(1, &depthRenderbuffer);
	glBindRenderbuffer(GL_RENDERBUFFER, depthRenderbuffer);
	glRenderbufferStorage(GL_RENDERBUFFER, GL_DEPTH_COMPONENT16, backingWidth, backingHeight);
	glFramebufferRenderbuffer(GL_FRAMEBUFFER, GL_DEPTH_ATTACHMENT, GL_RENDERBUFFER, depthRenderbuffer);
	
	if(glCheckFramebufferStatus(GL_FRAMEBUFFER) != GL_FRAMEBUFFER_COMPLETE) 
	{
		NSLog(@"Failure with framebuffer generation");
		return NO;
	}
#endif
	
    glViewport(0, 0, backingWidth, backingHeight);
	
    glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
    glClear(GL_COLOR_BUFFER_BIT);
	
	return YES;
}

- (BOOL)presentFramebuffer;
{
    BOOL success = FALSE;
    
    if (context)
    {
		//      [EAGLContext setCurrentContext:context];
#ifdef MSAA
		glBindFramebuffer(GL_READ_FRAMEBUFFER_APPLE, msaaFramebuffer); 
		glBindFramebuffer(GL_DRAW_FRAMEBUFFER_APPLE, viewFramebuffer);
		
		glResolveMultisampleFramebufferAPPLE();
#endif
        glBindRenderbuffer(GL_RENDERBUFFER, viewRenderbuffer);
        
        success = [context presentRenderbuffer:GL_RENDERBUFFER];
		
#ifdef MSAA
		glBindFramebuffer(GL_FRAMEBUFFER, msaaFramebuffer); 
#endif
    }
    
    return success;
}

- (void)switchToDisplayFramebuffer;
{
	glViewport(0, 0, backingWidth, backingHeight);

#ifdef MSAA
	glBindFramebuffer(GL_FRAMEBUFFER, msaaFramebuffer); 
#else
	glBindFramebuffer(GL_FRAMEBUFFER, viewFramebuffer);
#endif
}

- (void)destroyFramebuffer;
{	
	if (viewFramebuffer)
	{
		glDeleteFramebuffers(1, &viewFramebuffer);
		viewFramebuffer = 0;
	}
	
	if (viewRenderbuffer)
	{
		glDeleteRenderbuffers(1, &viewRenderbuffer);
		viewRenderbuffer = 0;
	}

	if (depthRenderbuffer)
	{
		glDeleteRenderbuffers(1, &depthRenderbuffer);
		depthRenderbuffer = 0;
	}
}



/*
// Only override drawRect: if you perform custom drawing.
// An empty implementation adversely affects performance during animation.
- (void)drawRect:(CGRect)rect {
    // Drawing code.
}
*/

- (void)dealloc {
    [super dealloc];
}

@synthesize offscreenRenderTexture, offscreenFramebuffer;

@end
