//
//  DetailViewController.m
//  ShaderExamples
//
//  Created by Brad Larson on 11/13/2010.
//  Copyright 2010 SonoPlot, Inc. All rights reserved.
//

#import "GLViewController.h"
#import "RootViewController.h"

#import "teapot.h"

#define DEBUG 1

@interface GLViewController ()
@property (nonatomic, retain) UIPopoverController *popoverController;
@end



@implementation GLViewController

#pragma mark -
#pragma mark Initialization and teardown

- (void)viewDidLoad 
{
	currentCalculatedMatrix = CATransform3DIdentity;
	currentCalculatedMatrix = CATransform3DTranslate(currentCalculatedMatrix, 0.0, -0.2, 0.0);
	currentCalculatedMatrix = CATransform3DScale(currentCalculatedMatrix, 4.5, 4.5 * (320.0/480.0), 4.5);

	lightDirection[0] = 0.0;
	lightDirection[1] = -0.7071;
	lightDirection[2] = -0.7071;
	
	hasGeneratedCubeMap = NO;
	
	plainDisplayProgram = [[GLProgram alloc] initWithVertexShaderFilename:@"PlainDisplay" fragmentShaderFilename:@"PlainDisplay"];
	[plainDisplayProgram addAttribute:@"position"];
	if (![plainDisplayProgram link])
	{
		NSLog(@"Plain display shader link failed");
		NSString *progLog = [plainDisplayProgram programLog];
		NSLog(@"Program Log: %@", progLog); 
		NSString *fragLog = [plainDisplayProgram fragmentShaderLog];
		NSLog(@"Frag Log: %@", fragLog);
		NSString *vertLog = [plainDisplayProgram vertexShaderLog];
		NSLog(@"Vert Log: %@", vertLog);
		[plainDisplayProgram release];
		plainDisplayProgram = nil;
	}
	plainDisplayPositionAttribute = [plainDisplayProgram attributeIndex:@"position"];
	plainDisplayModelViewMatrix = [plainDisplayProgram uniformIndex:@"modelViewProjMatrix"];

	simpleLightingProgram = [[GLProgram alloc] initWithVertexShaderFilename:@"SimpleLighting" fragmentShaderFilename:@"SimpleLighting"];
	[simpleLightingProgram addAttribute:@"position"];
	[simpleLightingProgram addAttribute:@"normal"];
	if (![simpleLightingProgram link])
	{
		NSLog(@"Simple lighting link failed");
		NSString *progLog = [simpleLightingProgram programLog];
		NSLog(@"Program Log: %@", progLog); 
		NSString *fragLog = [simpleLightingProgram fragmentShaderLog];
		NSLog(@"Frag Log: %@", fragLog);
		NSString *vertLog = [simpleLightingProgram vertexShaderLog];
		NSLog(@"Vert Log: %@", vertLog);
		[simpleLightingProgram release];
		simpleLightingProgram = nil;
	}
	simpleLightingPositionAttribute = [simpleLightingProgram attributeIndex:@"position"];
	simpleLightingNormalAttribute = [simpleLightingProgram attributeIndex:@"normal"];
	simpleLightingModelViewMatrix = [simpleLightingProgram uniformIndex:@"modelViewProjMatrix"];
	simpleLightingLightDirection = [simpleLightingProgram uniformIndex:@"lightDirection"];


	brickShaderProgram = [[GLProgram alloc] initWithVertexShaderFilename:@"BrickShader" fragmentShaderFilename:@"BrickShader"];
	[brickShaderProgram addAttribute:@"position"];
	[brickShaderProgram addAttribute:@"normal"];
	if (![brickShaderProgram link])
	{
		NSLog(@"Brick shader link failed");
		NSString *progLog = [brickShaderProgram programLog];
		NSLog(@"Program Log: %@", progLog); 
		NSString *fragLog = [brickShaderProgram fragmentShaderLog];
		NSLog(@"Frag Log: %@", fragLog);
		NSString *vertLog = [brickShaderProgram vertexShaderLog];
		NSLog(@"Vert Log: %@", vertLog);
		[brickShaderProgram release];
		brickShaderProgram = nil;
	}
	brickPositionAttribute = [brickShaderProgram attributeIndex:@"position"];
	brickNormalAttribute = [brickShaderProgram attributeIndex:@"normal"];	
	brickModelViewMatrix = [brickShaderProgram uniformIndex:@"modelViewProjMatrix"];
	brickLightPosition = [brickShaderProgram uniformIndex:@"LightPosition"];
	brickColor = [brickShaderProgram uniformIndex:@"BrickColor"];
	brickMortarColor = [brickShaderProgram uniformIndex:@"MortarColor"];
	brickSize = [brickShaderProgram uniformIndex:@"BrickSize"];
	brickPercentage = [brickShaderProgram uniformIndex:@"BrickPct"];

	
	toonShaderProgram = [[GLProgram alloc] initWithVertexShaderFilename:@"Toon" fragmentShaderFilename:@"Toon"];
	[toonShaderProgram addAttribute:@"position"];
	[toonShaderProgram addAttribute:@"normal"];
	if (![toonShaderProgram link])
	{
		NSLog(@"Toon shader link failed");
		NSString *progLog = [toonShaderProgram programLog];
		NSLog(@"Program Log: %@", progLog); 
		NSString *fragLog = [toonShaderProgram fragmentShaderLog];
		NSLog(@"Frag Log: %@", fragLog);
		NSString *vertLog = [toonShaderProgram vertexShaderLog];
		NSLog(@"Vert Log: %@", vertLog);
		[toonShaderProgram release];
		toonShaderProgram = nil;
	}
	toonPositionAttribute = [toonShaderProgram attributeIndex:@"position"];
	toonNormalAttribute = [toonShaderProgram attributeIndex:@"normal"];	
	toonModelViewMatrix = [toonShaderProgram uniformIndex:@"modelViewProjMatrix"];
	toonDiffuseColor = [toonShaderProgram uniformIndex:@"DiffuseColor"];
	toonPhongColor = [toonShaderProgram uniformIndex:@"PhongColor"];
	toonEdge = [toonShaderProgram uniformIndex:@"Edge"];
	toonPhong = [toonShaderProgram uniformIndex:@"Phong"];

	
	reflectionShaderProgram = [[GLProgram alloc] initWithVertexShaderFilename:@"Reflection" fragmentShaderFilename:@"Reflection"];
	[reflectionShaderProgram addAttribute:@"position"];
	[reflectionShaderProgram addAttribute:@"normal"];
	if (![reflectionShaderProgram link])
	{
		NSLog(@"Reflection shader link failed");
		NSString *progLog = [reflectionShaderProgram programLog];
		NSLog(@"Program Log: %@", progLog); 
		NSString *fragLog = [reflectionShaderProgram fragmentShaderLog];
		NSLog(@"Frag Log: %@", fragLog);
		NSString *vertLog = [reflectionShaderProgram vertexShaderLog];
		NSLog(@"Vert Log: %@", vertLog);
		[reflectionShaderProgram release];
		reflectionShaderProgram = nil;
	}
	reflectionPositionAttribute = [reflectionShaderProgram attributeIndex:@"position"];
	reflectionNormalAttribute = [reflectionShaderProgram attributeIndex:@"normal"];	
	reflectionModelViewMatrix = [reflectionShaderProgram uniformIndex:@"modelViewProjMatrix"];
	reflectionLightDirection = [reflectionShaderProgram uniformIndex:@"lightDirection"];
	reflectionModel = [reflectionShaderProgram uniformIndex:@"Model"];
	reflectionEyePosition = [reflectionShaderProgram uniformIndex:@"EyePosition"];
	reflectionCubeMap = [reflectionShaderProgram uniformIndex:@"cubeMap"];
	
	[self renderByRotatingAroundX:0.0 rotatingAroundY:0.0];
    [super viewDidLoad];
}

- (void)dealloc 
{
	[plainDisplayProgram release];
    [popoverController release];
    [toolbar release];
    
    [super dealloc];
}

#pragma mark -
#pragma mark Split view support

- (void)splitViewController: (UISplitViewController*)svc willHideViewController:(UIViewController *)aViewController withBarButtonItem:(UIBarButtonItem*)barButtonItem forPopoverController: (UIPopoverController*)pc 
{
    
    barButtonItem.title = @"Root List";
    NSMutableArray *items = [[toolbar items] mutableCopy];
    [items insertObject:barButtonItem atIndex:0];
    [toolbar setItems:items animated:YES];
    [items release];
    self.popoverController = pc;
}

- (void)splitViewController: (UISplitViewController*)svc willShowViewController:(UIViewController *)aViewController invalidatingBarButtonItem:(UIBarButtonItem *)barButtonItem 
{
    
    NSMutableArray *items = [[toolbar items] mutableCopy];
    [items removeObjectAtIndex:0];
    [toolbar setItems:items animated:YES];
    [items release];
    self.popoverController = nil;
}

#pragma mark -
#pragma mark Rotation support

// Ensure that the view controller supports rotation and that the split view can therefore show in both portrait and landscape.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation 
{
    return YES;
}

#pragma mark -
#pragma mark OpenGL rendering

- (void)renderByRotatingAroundX:(float)xRotation rotatingAroundY:(float)yRotation;
{
// 	glEnable(GL_CULL_FACE);
//	glCullFace(GL_FRONT);
	
    glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	GLfloat currentModelViewMatrix[16];	
	
	// Perform incremental rotation based on current angles in X and Y	
	if ((xRotation != 0.0) || (yRotation != 0.0))
	{
		GLfloat totalRotation = sqrt(xRotation*xRotation + yRotation*yRotation);
		
		CATransform3D temporaryMatrix = CATransform3DRotate(currentCalculatedMatrix, totalRotation * M_PI / 180.0, 
															((xRotation/totalRotation) * currentCalculatedMatrix.m12 + (yRotation/totalRotation) * currentCalculatedMatrix.m11),
															((xRotation/totalRotation) * currentCalculatedMatrix.m22 + (yRotation/totalRotation) * currentCalculatedMatrix.m21),
															((xRotation/totalRotation) * currentCalculatedMatrix.m32 + (yRotation/totalRotation) * currentCalculatedMatrix.m31));
		if ((temporaryMatrix.m11 >= -100.0) && (temporaryMatrix.m11 <= 100.0))
			currentCalculatedMatrix = temporaryMatrix;
	}
	else
	{
	}
	
	// Draw the teapot model
	
	[self convert3DTransform:&currentCalculatedMatrix toMatrix:currentModelViewMatrix];
	
	switch(currentShaderDisplayMode)
	{
		case PLAINDISPLAYSHADER:
		{
			[plainDisplayProgram use];

			glUniformMatrix4fv(plainDisplayModelViewMatrix, 1, 0, currentModelViewMatrix);
			glVertexAttribPointer(plainDisplayPositionAttribute, 3, GL_FLOAT, 0, 0, teapot_vertices);
			glEnableVertexAttribArray(plainDisplayPositionAttribute);
		}; break;
		case SIMPLELIGHTINGSHADER:
		{
			[simpleLightingProgram use];

			glUniformMatrix4fv(simpleLightingModelViewMatrix, 1, 0, currentModelViewMatrix);
			glUniform3fv(simpleLightingLightDirection, 1, lightDirection);
			glVertexAttribPointer(simpleLightingPositionAttribute, 3, GL_FLOAT, 0, 0, teapot_vertices);
			glEnableVertexAttribArray(simpleLightingPositionAttribute);
			glVertexAttribPointer(simpleLightingNormalAttribute, 3, GL_FLOAT, 0, 0, teapot_normals);
			glEnableVertexAttribArray(simpleLightingNormalAttribute);
			
//			[simpleLightingProgram validate];
		}; break;
		case BRICKSHADER:
		{
			[brickShaderProgram use];

			glUniformMatrix4fv(brickModelViewMatrix, 1, 0, currentModelViewMatrix);
			glUniform3fv(brickLightPosition, 1, lightDirection);
			glUniform2f(brickSize, 0.05, 0.05);
			glUniform2f(brickPercentage, 0.9, 0.9);
			glUniform3f(brickColor, 1.0, 0.0, 0.0);
			glUniform3f(brickMortarColor, 1.0, 1.0, 1.0);
			
			glVertexAttribPointer(brickPositionAttribute, 3, GL_FLOAT, 0, 0, teapot_vertices);
			glEnableVertexAttribArray(brickPositionAttribute);
			glVertexAttribPointer(brickNormalAttribute, 3, GL_FLOAT, 0, 0, teapot_normals);
			glEnableVertexAttribArray(brickNormalAttribute);
			
		}; break;
		case TOONSHADER:
		{
			[toonShaderProgram use];
			
			glUniformMatrix4fv(toonModelViewMatrix, 1, 0, currentModelViewMatrix);
			glUniform3f(toonDiffuseColor, 1.0, 1.0, 0.0);
			glUniform3f(toonPhongColor, 0.5, 0.5, 0.0);
			glUniform1f(toonEdge, 0.4);
			glUniform1f(toonPhong, 0.1);
			
			glVertexAttribPointer(toonPositionAttribute, 3, GL_FLOAT, 0, 0, teapot_vertices);
			glEnableVertexAttribArray(toonPositionAttribute);
			glVertexAttribPointer(toonNormalAttribute, 3, GL_FLOAT, 0, 0, teapot_normals);
			glEnableVertexAttribArray(toonNormalAttribute);
			
		}; break;
		case REFLECTIONSHADER:
		{
			//reflectionModel
			[self generateCubeMap];
			
			[reflectionShaderProgram use];
			
			glActiveTexture(GL_TEXTURE0);
			glBindTexture(GL_TEXTURE_CUBE_MAP, [(GLView *)self.view offscreenRenderTexture]);
			glUniform1i(reflectionCubeMap, 0);
			glUniformMatrix4fv(reflectionModelViewMatrix, 1, 0, currentModelViewMatrix);
			glUniform3f(reflectionEyePosition, 0.0, 0.0, 10.2);
			glUniform3fv(reflectionLightDirection, 1, lightDirection);

			glVertexAttribPointer(reflectionPositionAttribute, 3, GL_FLOAT, 0, 0, teapot_vertices);
			glEnableVertexAttribArray(reflectionPositionAttribute);
			glVertexAttribPointer(reflectionNormalAttribute, 3, GL_FLOAT, 0, 0, teapot_normals);
			glEnableVertexAttribArray(reflectionNormalAttribute);
		}; break;
	}
	
	// Draw teapot. The new_teapot_indicies array is an RLE (run-length encoded) version of the teapot_indices array in teapot.h
	for(int i = 0; i < num_teapot_indices; i += new_teapot_indicies[i] + 1)
	{
		glDrawElements(GL_TRIANGLE_STRIP, new_teapot_indicies[i], GL_UNSIGNED_SHORT, &new_teapot_indicies[i + 1]);
	}
	
    [(GLView *)self.view presentFramebuffer];
}

- (void)convert3DTransform:(CATransform3D *)transform3D toMatrix:(GLfloat *)matrix;
{
	//	struct CATransform3D
	//	{
	//		CGFloat m11, m12, m13, m14;
	//		CGFloat m21, m22, m23, m24;
	//		CGFloat m31, m32, m33, m34;
	//		CGFloat m41, m42, m43, m44;
	//	};
	
	matrix[0] = (GLfloat)transform3D->m11;
	matrix[1] = (GLfloat)transform3D->m12;
	matrix[2] = (GLfloat)transform3D->m13;
	matrix[3] = (GLfloat)transform3D->m14;
	matrix[4] = (GLfloat)transform3D->m21;
	matrix[5] = (GLfloat)transform3D->m22;
	matrix[6] = (GLfloat)transform3D->m23;
	matrix[7] = (GLfloat)transform3D->m24;
	matrix[8] = (GLfloat)transform3D->m31;
	matrix[9] = (GLfloat)transform3D->m32;
	matrix[10] = (GLfloat)transform3D->m33;
	matrix[11] = (GLfloat)transform3D->m34;
	matrix[12] = (GLfloat)transform3D->m41;
	matrix[13] = (GLfloat)transform3D->m42;
	matrix[14] = (GLfloat)transform3D->m43;
	matrix[15] = (GLfloat)transform3D->m44;
}

- (void)generateCubeMap;
{
	// Draw flat brick texture
    static const GLfloat squareVertices[] = {
        -1.0f, -1.0f, 0.0f,
        1.0f, -1.0f, 0.0f,
        -1.0f,  1.0f, 0.0f,
        1.0f,  1.0f, 0.0f,
    };
	
	static const GLfloat squareNormals[] = {
        0.0f, 1.0f, 0.0f,
        0.0f, 1.0f, 0.0f,
        0.0f, 1.0f, 0.0f,
        0.0f, 1.0f, 0.0f,
    };
	
	[brickShaderProgram use];
	
	GLfloat currentModelViewMatrix[16];

	CATransform3D currentBrickMatrix = CATransform3DMakeTranslation(0.0, -0.0, 0.0);
	currentBrickMatrix = CATransform3DRotate(currentBrickMatrix, 1.0 * M_PI / 8.0, 1.0, 0.0, 0.0);
	currentBrickMatrix = CATransform3DScale(currentBrickMatrix, 4.5, 4.5, 4.5);
	
	//	CATransform3D currentBrickMatrix = CATransform3DMakeRotation(3.0 * M_PI / 8.0, 1.0, 0.0, 0.0);
	//	currentBrickMatrix = CATransform3DTranslate(currentBrickMatrix, 0.0f, -3.5, 0.0);
	
//	glUniform3f(brickLightPosition, 0.0, 0.0, 0.0);
//	glUniform3f(brickLightPosition, 1.0, 1.0, 0.5);
	glUniform3fv(brickLightPosition, 1, lightDirection);
	glUniform2f(brickSize, 0.05, 0.05);
	glUniform2f(brickPercentage, 0.9, 0.9);
	glUniform3f(brickColor, 1.0, 0.0, 0.0);
	glUniform3f(brickMortarColor, 1.0, 1.0, 1.0);
	
	glVertexAttribPointer(brickPositionAttribute, 3, GL_FLOAT, 0, 0, squareVertices);
	glEnableVertexAttribArray(brickPositionAttribute);
	glVertexAttribPointer(brickNormalAttribute, 3, GL_FLOAT, 0, 0, squareNormals);
	glEnableVertexAttribArray(brickNormalAttribute);
	
	
	if (!hasGeneratedCubeMap)
	{
		GLView *glView = (GLView *)self.view;
		
		glBindFramebuffer(GL_FRAMEBUFFER, [glView offscreenFramebuffer]);
		glViewport(0, 0, 512, 512);
		for (unsigned int face = 0; face < 6; face++)
		{
			GLenum textureTarget = GL_TEXTURE_CUBE_MAP_POSITIVE_X + face;
			glFramebufferTexture2D(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, textureTarget, [glView offscreenRenderTexture], 0);
			
            glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
			// Adjust model view matrix for each view
			currentBrickMatrix = CATransform3DMakeTranslation(-0.5, -0.5, 0.0);

			switch (textureTarget)
			{
				case GL_TEXTURE_CUBE_MAP_POSITIVE_X:
				{
					currentBrickMatrix = CATransform3DRotate(currentBrickMatrix, M_PI / 2.0, 0.0, 1.0, 0.0);
				}; break;
				case GL_TEXTURE_CUBE_MAP_NEGATIVE_X:
				{
					currentBrickMatrix = CATransform3DRotate(currentBrickMatrix, 3.0 * M_PI / 2.0, 0.0, 1.0, 0.0);
				}; break;
				case GL_TEXTURE_CUBE_MAP_POSITIVE_Y:
				{
					currentBrickMatrix = CATransform3DRotate(currentBrickMatrix, 0.0, 1.0, 0.0, 0.0);
				}; break;
				case GL_TEXTURE_CUBE_MAP_NEGATIVE_Y:
				{
					currentBrickMatrix = CATransform3DMakeTranslation(0.0, -20.5, 0.0);
					currentBrickMatrix = CATransform3DRotate(currentBrickMatrix, M_PI, 1.0, 0.0, 0.0);
				}; break;
				case GL_TEXTURE_CUBE_MAP_POSITIVE_Z:
				{
					currentBrickMatrix = CATransform3DRotate(currentBrickMatrix, M_PI / 2.0, 1.0, 0.0, 0.0);
				}; break;
				case GL_TEXTURE_CUBE_MAP_NEGATIVE_Z:
				{
					currentBrickMatrix = CATransform3DRotate(currentBrickMatrix, 3.0 * M_PI / 2.0, 1.0, 0.0, 0.0);
				}; break;
			}

			currentBrickMatrix = CATransform3DScale(currentBrickMatrix, 4.5, 4.5, 4.5);
			[self convert3DTransform:&currentBrickMatrix toMatrix:currentModelViewMatrix];
			glUniformMatrix4fv(brickModelViewMatrix, 1, 0, currentModelViewMatrix);
			
			glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
			
		}

		[glView switchToDisplayFramebuffer];
		
		hasGeneratedCubeMap = YES;
	}

//	currentBrickMatrix = CATransform3DMakeTranslation(0.0, -0.5, 0.0);
//	currentBrickMatrix = CATransform3DRotate(currentBrickMatrix, M_PI / 2.0, 0.0, 1.0, 0.0);

//	currentBrickMatrix = CATransform3DMakeTranslation(0.0, 0.0, 0.5);
	//currentBrickMatrix = CATransform3DIdentity;
	
	currentBrickMatrix = CATransform3DMakeTranslation(-1.5, -1.5, 0.0);
    currentBrickMatrix.m33 = 0.5;
    currentBrickMatrix.m34 = 0.5;
	currentBrickMatrix = CATransform3DRotate(currentBrickMatrix, 3.0 * M_PI / 8.0, 1.0, 0.0, 0.0);
	currentBrickMatrix = CATransform3DScale(currentBrickMatrix, 4.5, 4.5, 4.5);

	[self convert3DTransform:&currentBrickMatrix toMatrix:currentModelViewMatrix];
	glUniformMatrix4fv(brickModelViewMatrix, 1, 0, currentModelViewMatrix);
	glUniform3f(brickLightPosition, 0.0, 0.0, 1.0);

	glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
}

#pragma mark -
#pragma mark Touch-handling methods

- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event
{
    NSMutableSet *currentTouches = [[[event touchesForView:self.view] mutableCopy] autorelease];
    [currentTouches minusSet:touches];
	
	// New touches are not yet included in the current touches for the view
	lastMovementPosition = [[touches anyObject] locationInView:self.view];
}

- (void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event;
{
	CGPoint currentMovementPosition = [[touches anyObject] locationInView:self.view];
	[self renderByRotatingAroundX:(lastMovementPosition.x - currentMovementPosition.x) rotatingAroundY:(lastMovementPosition.y - currentMovementPosition.y)];
	lastMovementPosition = currentMovementPosition;
}

- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event 
{
	NSMutableSet *remainingTouches = [[[event touchesForView:self.view] mutableCopy] autorelease];
    [remainingTouches minusSet:touches];
	
	lastMovementPosition = [[remainingTouches anyObject] locationInView:self.view];
	
	if ([[touches anyObject] tapCount] > 1)
	{
		CGPoint currentPoint = [[touches anyObject] locationInView:self.view];
		lightDirection[0] = (currentPoint.x / self.view.bounds.size.width);
		lightDirection[1] = 1.0 - (currentPoint.y / self.view.bounds.size.height);
		lightDirection[2] = 1.0;
		GLfloat normalFactor = sqrt(lightDirection[0] * lightDirection[0] + lightDirection[1] * lightDirection[1] + lightDirection[2] * lightDirection[2]);
		lightDirection[0] = lightDirection[0] / normalFactor;
		lightDirection[1] = lightDirection[1] / normalFactor;
		lightDirection[2] = -lightDirection[2] / normalFactor;
		
		[self renderByRotatingAroundX:0.0 rotatingAroundY:0.0];
	}
}

- (void)touchesCancelled:(NSSet *)touches withEvent:(UIEvent *)event 
{
	// Handle touches canceled the same as as a touches ended event
    [self touchesEnded:touches withEvent:event];
}

#pragma mark -
#pragma mark Accessors

@synthesize toolbar, popoverController;
@synthesize currentShaderDisplayMode;

- (void)setCurrentShaderDisplayMode:(ShaderDisplayMode)newValue 
{
    if (currentShaderDisplayMode != newValue) 
	{
		currentShaderDisplayMode = newValue;
		[self renderByRotatingAroundX:0.0 rotatingAroundY:0.0];
		
		NSLog(@"Switched renderer");
    }
	
    if (self.popoverController != nil) 
	{
        [self.popoverController dismissPopoverAnimated:YES];
    }        
}



@end
